@@ -492,7 +492,7 @@ int read_riskspec(char *filename) {
   FILE *input;
 
   /* parse the spec */
-  if( (input = fopen(filename, "r")) ) {
+  if( (input = fopen(filename, "rb")) ) {
 
     if( parse_risk_spec(input) != 0 ) {
       fclose(input);
@@ -774,7 +774,7 @@ int main(int argc, char **argv) {
      or if none provided read stdin */
   while( (optind > -1) && *(argv + optind) ) {
     /* if it's a filename, process it */
-    if( (input = fopen(argv[optind], "r")) ) {
+    if( (input = fopen(argv[optind], "rb")) ) {
       inputfile = argv[optind];
       options |= (1<<INPUT_FROM_CMDLINE);
 
@@ -802,20 +802,22 @@ int main(int argc, char **argv) {
     optind++;
   }
   /* in case no files were specified, get input from stdin */
-  if( !(options & (1<<INPUT_FROM_CMDLINE)) ) {
+  if( !(options & (1<<INPUT_FROM_CMDLINE)) &&
+      (input = fdopen(fileno(stdin), "rb")) ) {
 
     if( options & (1<<OPTION_DEBUG) ) {
       fprintf(stdout, "taking input from stdin\n");
     }
 
     if( !(options & (1<<OPTION_I18N)) ) {
-      b_process_file(stdin, line_fun);
+      b_process_file(input, line_fun);
     } else {
 #if defined HAVE_MBRTOWC
-      w_b_process_file(stdin, line_fun);
+      w_b_process_file(input, line_fun);
 #endif
     }
 
+    fclose(input);
   }
   
   if( postprocess_fun ) { (*postprocess_fun)(); }
