@@ -89,6 +89,7 @@ char *print_model_options(options_t opt, char *buf) {
   strcat(buf, MOPTION(M_OPTION_CHAR_GRAPH, opt));
   strcat(buf, MOPTION(M_OPTION_CHAR_ADP, opt));
   strcat(buf, MOPTION(M_OPTION_CHAR_CEF, opt));
+  strcat(buf, MOPTION(M_OPTION_CHAR_CHAR, opt));
   strcat(buf, MOPTION(M_OPTION_HEADERS, opt));
   strcat(buf, MOPTION(M_OPTION_PLAIN, opt));
   strcat(buf, MOPTION(M_OPTION_NOPLAIN, opt));
@@ -212,6 +213,7 @@ error_code_t sanitize_model_options(options_t *mopt, category_t *cat) {
   mask = 
     (1<<M_OPTION_CHAR_ALPHA)|
     (1<<M_OPTION_CHAR_ALNUM)|
+    (1<<M_OPTION_CHAR_CHAR)|
     (1<<M_OPTION_CHAR_GRAPH)|
     (1<<M_OPTION_CHAR_ADP)|
     (1<<M_OPTION_CHAR_CEF);
@@ -427,6 +429,7 @@ void init_category(category_t *cat) {
     cat->score_div = 0.0;
     cat->score_s2 = 0.0;
     cat->score_shannon = 0.0;
+    cat->score_exp = 0.0;
     cat->shannon = 0.0;
     cat->alpha = 0.0;
     cat->beta = 0.0;
@@ -465,6 +468,7 @@ bool_t create_category_hash(category_t *cat, FILE *input, int protf) {
 	(byte_t *)MMAP(0, sizeof(c_item_t) * cat->max_tokens + 
 		       cat->mmap_offset,
 		       protf, MAP_SHARED, fileno(input), 0);
+      if( cat->mmap_start == MAP_FAILED ) { cat->mmap_start = NULL; }
       if( cat->mmap_start ) {
 	cat->hash = (c_item_t *)(cat->mmap_start + cat->mmap_offset);
 	MADVISE(cat->hash, sizeof(c_item_t) * cat->max_tokens, 
@@ -877,32 +881,18 @@ confidence_t gamma_pvalue(category_t *cat, double obs) {
 /***********************************************************
  * FILE MANAGEMENT FUNCTIONS                               *
  ***********************************************************/
-
-error_code_t explicit_load_category(category_t *cat, char *openf, int protf) {
-  hash_count_t i, j;
+error_code_t load_category_header(FILE *input, category_t *cat) {
   char buf[MAGIC_BUFSIZE];
   char scratchbuf[MAGIC_BUFSIZE];
-
   short int shint_val;
   long int lint_val1, lint_val2, lint_val3;
 
-  FILE *input;
-
-  /* this is needed in case we try to open with write permissions,
-     which would otherwise create the file */
-
-  input = fopen(cat->fullfilename, "rb");
-  if( input && (strcmp(openf, "rb") != 0) ) {
-    input = freopen(cat->fullfilename, openf, input);
-  }
-
   if( input ) {
     if( !fgets(buf, MAGIC_BUFSIZE, input) ||
 	strncmp(buf, MAGIC1, MAGIC1_LEN) ) {
       errormsg(E_ERROR,
 	      "not a dbacl " SIGNATURE " category file [%s]\n", 
 	      cat->fullfilename);
-      fclose(input);
       return 0;
     } 
 
@@ -912,7 +902,6 @@ error_code_t explicit_load_category(category_t *cat, char *openf, int protf) {
 	(sscanf(buf, MAGIC2_i, &cat->divergence, &cat->logZ, 
 		&shint_val, scratchbuf) < 4) ) {
       errormsg(E_ERROR, "bad category file [2]\n");
-      fclose(input);
       return 0;
     }
     cat->max_order = (token_order_t)shint_val;
@@ -931,7 +920,6 @@ error_code_t explicit_load_category(category_t *cat, char *openf, int protf) {
 		&lint_val2,
 		&lint_val3) < 4) ) {
       errormsg(E_ERROR, "bad category file [3]\n");
-      fclose(input);
       return 0;
     }
     cat->max_hash_bits = (token_order_t)shint_val;
@@ -947,7 +935,6 @@ error_code_t explicit_load_category(category_t *cat, char *openf, int protf) {
 		&cat->alpha, &cat->beta,
 		&cat->mu, &cat->s2) < 5) ) {
       errormsg(E_ERROR, "bad category file [8]\n");
-      fclose(input);
       return 0;
     }
 
@@ -979,6 +966,7 @@ error_code_t explicit_load_category(category_t *cat, char *openf, int protf) {
     if( !(cat->m_options & (1<<M_OPTION_CHAR_ALPHA)) &&
 	!(cat->m_options & (1<<M_OPTION_CHAR_ALNUM)) &&
 	!(cat->m_options & (1<<M_OPTION_CHAR_CEF)) &&
+	!(cat->m_options & (1<<M_OPTION_CHAR_CHAR)) &&
 	!(cat->m_options & (1<<M_OPTION_CHAR_ADP)) &&
 	!(cat->m_options & (1<<M_OPTION_CHAR_GRAPH)) ) {
       if( cat->m_options & (1<<M_OPTION_MBOX_FORMAT) ) {
@@ -987,6 +975,32 @@ error_code_t explicit_load_category(category_t *cat, char *openf, int protf) {
 	cat->m_options |= (1<<M_OPTION_CHAR_ALPHA);
       }
     }
+    /* if we're here, success! */
+    return 1;
+  }
+  return 0;
+}
+
+
+error_code_t explicit_load_category(category_t *cat, char *openf, int protf) {
+  hash_count_t i, j;
+
+  FILE *input;
+
+  /* this is needed in case we try to open with write permissions,
+     which would otherwise create the file */
+
+  input = fopen(cat->fullfilename, "rb");
+  if( input && (strcmp(openf, "rb") != 0) ) {
+    input = freopen(cat->fullfilename, openf, input);
+  }
+
+  if( input ) {
+
+    if( !load_category_header(input, cat) ) {
+      fclose(input);
+      return 0;
+    }
 
     /* read character frequencies */
     i = ASIZE * ASIZE;
