@@ -143,6 +143,7 @@ typedef uint64_t u_int64_t;
 #endif
 
 #ifndef MADVISE
+#define MAP_FAILED ((void *)-1)
 #define MADVISE(x,y,z)
 #define MLOCK(x,y)
 #define MUNLOCK(x,y)
@@ -481,6 +482,7 @@ typedef u_int16_t digitized_weight_t;
 #define M_OPTION_CASEN                  6
 #define M_OPTION_CALCENTROPY            7
 #define M_OPTION_MULTINOMIAL            8
+#define M_OPTION_CHAR_CHAR              9
 #define M_OPTION_CHAR_ALPHA             10
 #define M_OPTION_CHAR_ALNUM             11
 #define M_OPTION_CHAR_GRAPH             12
@@ -523,9 +525,6 @@ typedef enum {gcUNDEF = 0, gcDISCARD, gcTOKEN, gcTOKEN_END, gcIGNORE} good_char_
 
 /* macros */
 
-#define CVT_BITS(x) ((x)/M_LN2)
-/* #define CVT_BITS(x) (x) */
-
 /* used for digitizing */
 #if defined DIGITIZE_LWEIGHTS
 
@@ -642,6 +641,7 @@ typedef enum {gcUNDEF = 0, gcDISCARD, gcTOKEN, gcTOKEN_END, gcIGNORE} good_char_
 #define MAGIC7_i  "# antiregex %s\n"
 #define MAGIC7_o  "# antiregex %s||%s\n"
 #define MAGIC7_wo "# antiregex %ls||%s\n"
+#define MAGIC9    "# binned_features %ld max_feature_count %ld\n" 
 #define RESTARTPOS 8
 #define MAGIC6    "#\n"
 #define MAGIC8_i  "# shannon %" FMT_scanf_score_t \
@@ -657,6 +657,10 @@ typedef enum {gcUNDEF = 0, gcDISCARD, gcTOKEN, gcTOKEN_END, gcIGNORE} good_char_
 
 #define MAGIC_ONLINE "# dbacl " SIGNATURE " online memory dump\n"
 
+#define MAGIC_DUMP "# lambda | dig_ref | count | id     | token\n"
+#define MAGIC_DUMPTBL_o "%9.3f %9.3f %7d %8lx "
+#define MAGIC_DUMPTBL_i "%f %f %d %lx "
+
 /* data structures */
 typedef struct {
   token_class_t cls: 4;
@@ -714,6 +718,7 @@ typedef struct {
   score_t score_div;
   score_t score_s2;
   score_t score_shannon;
+  score_t score_exp;
   score_t shannon;
   score_t alpha;
   score_t beta;
@@ -774,9 +779,9 @@ typedef struct {
     long used;
     off_t avail;
     byte_t *mmap_start;
-    size_t mmap_offset;
+    long mmap_offset;
     size_t mmap_length;
-    size_t mmap_cursor;
+    long mmap_cursor;
   } tmp;
   re_bitfield retype;
   token_order_t max_order;
@@ -786,6 +791,8 @@ typedef struct {
   hash_count_t max_tokens;
   token_count_t full_token_count;
   token_count_t unique_token_count;
+  token_count_t t_max;
+  token_count_t b_count;
   score_t logZ;
   score_t divergence;
   score_t shannon;
@@ -916,7 +923,7 @@ typedef struct {
 } MBOX_State;
 
 typedef enum {TEXT=1, XTAG, XTAGQUOTE, XTAGDQUOTE, XTAGPREQ, TAG, TAGQUOTE, TAGDQUOTE, TAGPREQ, CMNT, DISABLED} Xstate;
-typedef enum {ALT=1,SRC,SRC_NETLOC,SRC_NETLOC_PREFIX,SRC_NETLOC_PATH,SRC_NETLOC_SUFFIX,UNDEF,JSCRIPT,ASTYLE} Xattribute;
+typedef enum {ALT=1, SRC, SRC_NETLOC, SRC_NETLOC_PREFIX, SRC_NETLOC_PATH, SRC_NETLOC_SUFFIX, UNDEF, JSCRIPT, ASTYLE} Xattribute;
 typedef enum {xpDUMB=1, xpHTML, xpSMART} Xparser;
 typedef enum {SCRIPT=1,STYLE,COMMENT,NOFRAMES,NOEMBED,NOSCRIPT,NOLAYER,TITLE,VISIBLE} Xhide;
 
@@ -964,7 +971,7 @@ extern "C"
   /* these are defined in catfun.c */
   char *sanitize_path(char *in, char *extension);
   error_code_t sanitize_model_options(options_t *to, category_t *cat);
-  char *print_model_options(options_t opt, char *buf);
+  /*@shared@*/ char *print_model_options(options_t opt, /*@out@*/ char *buf);
   char *print_user_options(options_t opt, char *buf);
 
   void init_empirical(empirical_t *emp, hash_count_t dmt, hash_bit_count_t dmhb);
@@ -979,6 +986,7 @@ extern "C"
   c_item_t *find_in_category(category_t *cat, hash_value_t id);
   void init_purely_random_text_category(category_t *cat);
   error_code_t load_category(category_t *cat);
+  error_code_t load_category_header(FILE *input, category_t *cat);
   error_code_t open_category(category_t *cat);
   void reload_all_categories();
 
