@@ -355,6 +355,7 @@ void free_mbox_line_filter(MBOX_State *mbox) {
 token_type_t get_token_type(token_order_t o) {
   token_type_t tt;
   tt.order = o;
+  tt.mark = 0;
 
   if( (m_options & (1<<M_OPTION_MBOX_FORMAT)) &&
       !(m_options & (1<<M_OPTION_NGRAM_STRADDLE_NL)) ) {
@@ -383,12 +384,16 @@ token_type_t get_token_type(token_order_t o) {
       }
       break;
     case msUNDEF:
+      tt.cls = AMIN + 3;
+      break;
     case msBODY:
-      tt.cls =  AMIN + 3;
+      tt.cls = AMIN + 3;
       break;
     case msATTACH:
       tt.cls =  AMIN + 2;
       break;
+    default:
+      tt.cls = AMIN + 1;
     }
   } else {
     tt.cls = AMIN + 1;
