@@ -159,12 +159,12 @@ int adjust_tz(struct tm *tim, char *p) {
     {"Z", 0}
   };
 
-  while( *p && !isalpha(*p) && (*p != '+') && (*p != '-') ) { p++; }
+  while( *p && !isalpha((int)*p) && (*p != '+') && (*p != '-') ) { p++; }
   switch(*p) {
   case '+':
   case '-':
     if( *p == '+' ) { s = -1; } else { s = +1; }
-    while( *p && !isdigit(*p) ) { p++; }
+    while( *p && !isdigit((int)*p) ) { p++; }
     a = atoi(p);
     tim->tm_hour += s * (int)(a / 100);
     tim->tm_min += s * (int)(a % 100);
@@ -204,10 +204,10 @@ time_t cvt_date(token_delim_t *tok) {
   p = tok->begin;
   if( !p ) { return -1; }
 
-  while( *p && !isdigit(*p) ) { p++; }
+  while( *p && !isdigit((int)*p) ) { p++; }
   tim.tm_mday = atoi(p);
   if( (tim.tm_mday < 1) || (tim.tm_mday > 31) ) { return -1; }
-  while( *p && !isalpha(*p) ) { p++; }
+  while( *p && !isalpha((int)*p) ) { p++; }
   tim.tm_mon = -1;
   for(i = 0; i < sizeof(month_name)/sizeof(char*); i++) {
     if( strncasecmp(p, month_name[i], 3) == 0 ) {
@@ -216,32 +216,32 @@ time_t cvt_date(token_delim_t *tok) {
     }
   }
   if( (tim.tm_mon < 0) || (tim.tm_mon > 11) ) { return -1; }
-  while( *p && !isdigit(*p) ) { p++; }
+  while( *p && !isdigit((int)*p) ) { p++; }
   tim.tm_year = atoi(p) - 1900;
   if( (tim.tm_year < 0) || (tim.tm_year > 132) ) { return -1; }
-  while( *p && !isspace(*p) ) { p++; }
-  while( *p && !isdigit(*p) ) { p++; }
+  while( *p && !isspace((int)*p) ) { p++; }
+  while( *p && !isdigit((int)*p) ) { p++; }
   tim.tm_hour = atoi(p);
   if( (tim.tm_hour < 0) || (tim.tm_hour > 23) ) { return -1; }
   while( *p && (*p != ':') ) { p++; }
-  while( *p && !isdigit(*p) ) { p++; }
+  while( *p && !isdigit((int)*p) ) { p++; }
   tim.tm_min = atoi(p);
   if( (tim.tm_min < 0) || (tim.tm_min > 59) ) { return -1; }
   for(q = p; *q && (*q != ':'); q++);
   if( *q == ':' ) {
-    for(p = q; *p && !isdigit(*p); p++);
+    for(p = q; *p && !isdigit((int)*p); p++);
     tim.tm_sec = atoi(p);
     if( (tim.tm_sec < 0) || (tim.tm_sec > 59) ) { return -1; }
   } else {
     tim.tm_sec = 0;
   }
-  while( *p && isdigit(*p) ) { p++; }
+  while( *p && isdigit((int)*p) ) { p++; }
   tim.tm_isdst = 0;
   /* verify weekday if present - this must be done before the time zone
    * adjustment!
    */
-  for(q = tok->begin; *q && !isalnum(*q); q++); /* used later */
-  if( isalpha(*q) ) {
+  for(q = tok->begin; *q && !isalnum((int)*q); q++); /* used later */
+  if( isalpha((int)*q) ) {
     for(i = 0; i < sizeof(day_name)/sizeof(char*); i++) {
       if( strncasecmp(q, day_name[i], 3) == 0 ) {
 	if( tim.tm_wday != i ) {
@@ -269,6 +269,7 @@ hline_t *mkh_ignore(HEADER_State *head, hline_t *t, char *line) {
   memset(t, 0, sizeof(hline_t));
   t->tag = hltIGN;
   t->data.ign.line = line;
+  t->toffset = head->hdata.curline;
 
   if( (u_options & (1<<U_OPTION_DEBUG)) ) {
     fprintf(stdout, "IGNORED\n");
@@ -281,6 +282,7 @@ hline_t *mkh_return_path(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltRET;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_return(line, &t->data.ret.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -314,6 +316,7 @@ hline_t *mkh_received(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltRCV;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_received(line, &t->data.rcv.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -364,6 +367,7 @@ hline_t *mkh_reply_to(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltRPT;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_reply_to(line, &t->data.rpt.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -389,6 +393,7 @@ hline_t *mkh_from(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltFRM;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_from(line, &t->data.frm.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -414,6 +419,7 @@ hline_t *mkh_sender(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltSND;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_sender(line, &t->data.snd.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -439,6 +445,7 @@ hline_t *mkh_resent_reply_to(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltRRT;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_resent_reply_to(line, &t->data.rrt.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -464,6 +471,7 @@ hline_t *mkh_resent_from(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltRFR;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_from(line, &t->data.rfr.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -489,6 +497,7 @@ hline_t *mkh_resent_sender(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltRSN;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_resent_sender(line, &t->data.rsn.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -514,6 +523,7 @@ hline_t *mkh_date(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltDAT;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_date(line, &t->data.dat.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -551,6 +561,7 @@ hline_t *mkh_resent_date(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltRDT;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_resent_date(line, &t->data.rdt.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -576,6 +587,7 @@ hline_t *mkh_to(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltTO;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_to(line, &t->data.to.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -601,6 +613,7 @@ hline_t *mkh_resent_to(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltRTO;
+  t->toffset = head->hdata.curline;
   
   if( parse_822_resent_to(line, &t->data.rto.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -626,6 +639,7 @@ hline_t *mkh_cc(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltCC;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_cc(line, &t->data.cc.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -651,6 +665,7 @@ hline_t *mkh_resent_cc(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltRCC;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_resent_cc(line, &t->data.rcc.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -676,6 +691,7 @@ hline_t *mkh_bcc(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltBCC;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_bcc(line, &t->data.bcc.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -701,6 +717,7 @@ hline_t *mkh_resent_bcc(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltRBC;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_resent_bcc(line, &t->data.rbc.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -726,6 +743,7 @@ hline_t *mkh_message_id(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltMID;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_message_id(line, &t->data.mid.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -751,6 +769,7 @@ hline_t *mkh_resent_message_id(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltRID;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_resent_message_id(line, &t->data.rid.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -776,6 +795,7 @@ hline_t *mkh_in_reply_to(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltIRT;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_in_reply_to(line, &t->data.irt.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -801,6 +821,7 @@ hline_t *mkh_references(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltREF;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_references(line, &t->data.ref.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -826,6 +847,7 @@ hline_t *mkh_subject(HEADER_State *head, hline_t *t, char *line) {
 
   memset(t, 0, sizeof(hline_t));
   t->tag = hltSBJ;
+  t->toffset = head->hdata.curline;
 
   if( parse_822_subject(line, &t->data.sbj.rfc822) ) {
     t->state |= (1<<H_STATE_RFC822);
@@ -919,6 +941,66 @@ hline_t *head_push_header(HEADER_State *head, char *line) {
   return tmp;
 }
 
+/***********************************************************
+ * MORE UTILITY FUNCTIONS                                  *
+ ***********************************************************/
+char *unfold_token(char *buf, int buflen, char *tokb, char *toke, char delim) {
+  if( buf && tokb && toke ) { 
+    buflen--; /* need space for the NUL */
+    for(; (tokb < toke) && isspace((int)*tokb); tokb++); /* skip leading whitespace */
+    while( (tokb < toke) && (buflen > 0) ) {
+      switch(*tokb) {
+      case '\n':
+      case '\r':
+	for(; (tokb < toke) && isspace((int)*tokb); tokb++);
+	*buf++ = ' ';
+	buflen--;
+	break;
+      case '\0':
+	/* ignore */
+	break;
+      default:
+	if( *tokb != delim ) {
+	  *buf++ = *tokb;
+	  buflen--;
+	} else {
+	  *buf = '\0';
+	  return (++tokb);
+	}
+      }
+      tokb++;
+    }
+    *buf = '\0';
+    return tokb;
+  }
+  return NULL;
+}
+
+
+/*
+ * Assumes that buf contains a mailbox token or similar and uses
+ * heuristics to check for an email address. Makes sure such an email
+ * is bracketed <>. Returns true if email appears to exist.
+ */
+bool_t bracketize_mailbox(char *buf, int buflen) {
+  /* a group ends in ; othewise it's a mailbox */
+  char *p;
+  int n;
+  if( !strrchr(buf, ';') ) {
+    p = strrchr(buf, '<');
+    if(  !p || (p >= strrchr(buf, '>')) ) {
+      n = strlen(buf);
+      if( (n + 2) < buflen ) {
+	memmove(buf + 1, buf, n);
+	buf[0] = '<';
+	buf[n+1] = '>';
+	buf[n+2] = '\0';
+      }
+    }
+    return 1;
+  }
+  return 0;
+} 
 
 /***********************************************************
  * TESTING                                                 *
@@ -972,9 +1054,9 @@ int main(int argc, char **argv) {
 	break;
       } else {
 	if( pptextbuf && !isblank(*textbuf) ) {
+	  pptextbuf = head_append_hline(&head, textbuf);
 	  h = head_push_header(&head, pptextbuf);
 	}
-	pptextbuf = head_append_hline(&head, textbuf);
       }
   }
 
