@@ -25,12 +25,15 @@ typedef struct {
   hash_value_t id;
   weight_t lam[2];
   weight_t ref;
-} hx_item_t;
+  token_type_t typ;
+} PACK_STRUCTS cp_item_t;
 
 typedef struct {
-  int stage;
-  weight_t logZ[2];
-  hx_item_t *hash;
-} hypex_t;
+  category_t cat[2];
+  hash_bit_count_t max_hash_bits;
+  hash_count_t max_tokens;
+  token_count_t unique_token_count;
+  cp_item_t *hash;
+} category_pair_t;
 
 #endif
