@@ -668,7 +668,9 @@ static void sigwinch_handler (int sig) {
 static void rline_update(unsigned char *buf, int len, int col) {
   SLsmg_gotorc (SLtt_Screen_Rows - 1, 0); 
   SLsmg_set_color(2);
-  SLsmg_write_nchars ((char *) buf, len); 
+  if( buf ) {
+    SLsmg_write_nchars ((char *) buf, len); 
+  }
   SLsmg_erase_eol ();      
   SLsmg_gotorc (SLtt_Screen_Rows - 1, col);
   SLsmg_refresh ();     
@@ -676,44 +678,75 @@ static void rline_update(unsigned char *buf, int len, int col) {
 
 /* make sure to call free on the returned string once no longer needed */
 char *get_input_line(char *title, char *prefill) {
+
+#if (SLANG_VERSION >= 20000) /* libslang2 */
+
+  char *s = NULL;
+  char *buf;
+  unsigned int len = 0;
+  SLrline_Type *rli;
+
+  rli = SLrline_open(SLtt_Screen_Cols, SL_RLINE_BLINK_MATCH);
+  if( rli ) {
+    SLtt_set_cursor_visibility(1);
+    SLsmg_normal_video();
+    len = 255;
+    rline_update(NULL, len, strlen(title));
+    buf = SLrline_read_line(rli, title, &len);
+    if( buf ) {
+      s = strdup(buf);
+      add_history(s); 
+    }
+    SLfree(buf);
+    SLtt_set_cursor_visibility(0);
+  }
+
+#else /* libslang1 */
+
   char *s;
   SLang_RLine_Info_Type *rli;
 
   s = malloc(256 * sizeof(char));
-  rli = (SLang_RLine_Info_Type *)malloc(sizeof(SLang_RLine_Info_Type));
-  if( rli ) {
-    memset(rli, 0, sizeof(SLang_RLine_Info_Type));
-    rli->buf = (unsigned char *)s; /* buf is unsigned */
-    rli->buf_len = 255;
-    rli->tab = 8;
-    rli->dhscroll = 5;
-    rli->getkey = SLang_getkey;
-    rli->tt_goto_column = NULL;
-    rli->update_hook = rline_update;
-    rli->flags |= SL_RLINE_BLINK_MATCH;
-    rli->input_pending = SLang_input_pending;
-    SLang_init_readline(rli);
-    rli->edit_width = SLtt_Screen_Cols - 1;
-    rli->prompt = title;
+  if( s ) {
 
-    /* now prefill if necessary */
-    if( *prefill ) {
-      strncpy(rli->buf, (unsigned char *)prefill, 255); 
-      rli->buf[255] = '\0';
-      rli->point = strlen((char *)rli->buf);
-    } else {
-      *rli->buf = 0;
-      rli->point = 0;
+    rli = (SLang_RLine_Info_Type *)malloc(sizeof(SLang_RLine_Info_Type));
+    if( rli ) {
+      memset(rli, 0, sizeof(SLang_RLine_Info_Type));
+      rli->buf = (unsigned char *)s; /* buf is unsigned */
+      rli->buf_len = 255;
+      rli->tab = 8;
+      rli->dhscroll = 5;
+      rli->getkey = SLang_getkey;
+      rli->tt_goto_column = NULL;
+      rli->update_hook = rline_update;
+      rli->flags |= SL_RLINE_BLINK_MATCH;
+      rli->input_pending = SLang_input_pending;
+      SLang_init_readline(rli);
+      rli->edit_width = SLtt_Screen_Cols - 1;
+      rli->prompt = title;
+
+      /* now prefill if necessary */
+      if( *prefill ) {
+	strncpy(s, prefill, 255); 
+	rli->buf[255] = '\0';
+	rli->point = strlen((char *)rli->buf);
+      } else {
+	*rli->buf = 0;
+	rli->point = 0;
+      }
+
+      SLtt_set_cursor_visibility(1);
+      SLsmg_normal_video();
+      SLang_read_line(rli);
+      add_history(s); /* buf is unsigned */
+      SLtt_set_cursor_visibility(0);
+      free(rli);
     }
-
   }
 
-  SLtt_set_cursor_visibility(1);
-  SLsmg_normal_video();
-  SLang_read_line(rli);
-  add_history((char *)rli->buf); /* buf is unsigned */
-  SLtt_set_cursor_visibility(0);
-  return (char *)rli->buf; /* buf is unsigned */
+#endif
+
+  return s;
 }
 
 #endif
