@@ -73,16 +73,16 @@ char *skip_single_char(char *line, char what) {
 
 static __inline__
 char *skip_twodigit(char *line) {
-  if( !line || !isdigit(line[0]) || !isdigit(line[1]) ) { return NULL; }
+  if( !line || !isdigit((int)line[0]) || !isdigit((int)line[1]) ) { return NULL; }
   line += 2;
   return line;
 }
 
 static __inline__
 char *skip_onetwodigit(char *line) {
-  if( !line || !isdigit(line[0]) ) { return NULL; }
+  if( !line || !isdigit((int)line[0]) ) { return NULL; }
   line++;
-  if( isdigit(line[0]) ) {
+  if( isdigit((int)line[0]) ) {
     line++;
   }
   return line;
@@ -90,8 +90,8 @@ char *skip_onetwodigit(char *line) {
 
 static __inline__
 char *skip_fourdigit(char *line) {
-  if( !line || !isdigit(line[0]) || !isdigit(line[1]) ||
-      !isdigit(line[2]) || !isdigit(line[3]) ) { return NULL; }
+  if( !line || !isdigit((int)line[0]) || !isdigit((int)line[1]) ||
+      !isdigit((int)line[2]) || !isdigit((int)line[3]) ) { return NULL; }
   line += 4;
   return line;
 }
@@ -124,12 +124,12 @@ char *skip_string_list(char *line, char *list[], int n) {
 static __inline__
 char *skip_2822_fws(char *line) {
   if( !line ) { return NULL; }
-  while( !isblank(*line) ) {
-    if( !isspace(*line) ) { return NULL; }
+  while( !isblank((int)*line) ) {
+    if( !isspace((int)*line) ) { return NULL; }
     line++;
   }
   line++;
-  while( isblank(*line) ) { line++; }
+  while( isblank((int)*line) ) { line++; }
   return line;
 }
 
@@ -142,7 +142,7 @@ char *skip_2822_fws(char *line) {
  */
 static __inline__
 char *unskip_2822_fws(char *line) {
-  while( line && isspace(line[-1]) ) {
+  while( line && isspace((int)line[-1]) ) {
     line--;
   }
   return line;
@@ -153,7 +153,7 @@ char *skip_2822_cfws(char *line, int min) {
   char *tmp;
   if( !line ) { return NULL; }
   while(min-- > 0) {
-    if( isspace(*line ) ) {
+    if( isspace((int)*line ) ) {
       line = skip_2822_fws(line);
     } else if( *line == '(' ) {
       line = skip_recursive(line, '(', ')', '\\');
@@ -163,7 +163,7 @@ char *skip_2822_cfws(char *line, int min) {
   }
   /* below space is optional */
   while( line ) {
-    if( isspace(*line ) ) {
+    if( isspace((int)*line ) ) {
       tmp = skip_2822_fws(line);
       if( tmp ) { 
 	line = tmp; 
@@ -211,7 +211,8 @@ char *skip_2822_obs_list(char *line, token_delim_t *tok,
 }
 
 char *skip_2822_seq(char *line, token_delim_t *tok, 
-	       char *(*skip)(char *,token_delim_t *, options_t), char delim, options_t opt) {
+		    char *(*skip)(char *,token_delim_t *, options_t), 
+		    char delim, options_t opt) {
   char *tmp;
   BOT;
   line = (*skip)(line, NULL, opt);
@@ -226,7 +227,7 @@ char *skip_2822_seq(char *line, token_delim_t *tok,
 }
 
 static char rfc2822_atom[256] = {
-  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
+ -1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0, '!',   0, '#', '$', '%', '&','\'',   0,   0, '*', '+',   0, '-',   0, '/',
   '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',   0,   0,   0, '=',   0, '?',
@@ -532,9 +533,9 @@ char *skip_2822_item_value(char *line, token_delim_t *tok, options_t opt) {
 
 static __inline__
 char *skip_2822_item_name(char *line, token_delim_t *tok) {
-  if( !line || !isalpha(*line) ) { return NULL; }
+  if( !line || !isalpha((int)*line) ) { return NULL; }
   BOT;
-  while( isalnum(*line) || (*line == '-') ) {
+  while( isalnum((int)*line) || (*line == '-') ) {
     line++;
   }
   EOT;
@@ -589,7 +590,7 @@ char *skip_2822_day_of_week(char *line, options_t opt) {
   if( !line ) { return NULL; }
   if( (opt & (1<<O_2822_OBS)) ) {
     line = skip_2822_cfws(line, 0);
-  } else if( isblank(*line) ) {
+  } else if( isblank((int)*line) ) {
     line = skip_2822_fws(line);
   }
   line = skip_string_list(line, day_name, sizeof(day_name)/sizeof(char*));
@@ -604,7 +605,7 @@ char *skip_2822_day(char *line, options_t opt) {
   if( !line ) { return NULL; }
   if( (opt & (1<<O_2822_OBS)) ) {
     line = skip_2822_cfws(line, 0);
-  } else if( isblank(*line) ) {
+  } else if( isblank((int)*line) ) {
     line = skip_2822_fws(line);
   }
   line = skip_onetwodigit(line);
@@ -717,8 +718,8 @@ char *skip_2822_obs_zone_name(char *line) {
 static __inline__
 char *skip_2822_obs_zone(char *line) {
   if( !line ) { return NULL; }
-  if( isalpha(line[0]) ) {
-    if( !isalpha(line[1]) ) {
+  if( isalpha((int)line[0]) ) {
+    if( !isalpha((int)line[1]) ) {
       line++;
     } else {
       line = skip_2822_obs_zone_name(line);
@@ -779,7 +780,7 @@ char *skip_2822_obs_phrase(char *line, options_t opt) {
   if( !line ) { return NULL; }
   if( *line == '.') {
     tmp = line + 1;
-  } else if( isspace(*line) ) {
+  } else if( isspace((int)*line) ) {
     tmp = skip_2822_cfws(line, 1);
   } else {
     tmp = skip_2822_word(line, NULL, opt);
@@ -788,7 +789,7 @@ char *skip_2822_obs_phrase(char *line, options_t opt) {
     line = tmp;
     if( *line == '.') {
       tmp = line + 1;
-    } else if( isspace(*line) ) {
+    } else if( isspace((int)*line) ) {
       tmp = skip_2822_cfws(line, 1);
     } else {
       tmp = skip_2822_word(line, NULL, opt);
@@ -991,7 +992,7 @@ static __inline__
 char *skip_2822_end_of_line(char *line) {
   /* check for CRLF, but OS may have mangled/removed it */
   if( !line || (strlen(line) > 2) ) { return NULL; }
-  while( isspace(*line) ) { line++; }
+  while( isspace((int)*line) ) { line++; }
   return (*line) ? NULL : line;
 }
 
@@ -1175,10 +1176,10 @@ char *parse_2822_subject(char *line, parse_2822_txt_t *p, options_t opt) {
 
 static __inline__
 char *skip_2821_sub_domain(char *line, token_delim_t *tok, options_t opt) {
-  if( !line || !isalnum(*line) ) { return NULL; }
+  if( !line || !isalnum((int)*line) ) { return NULL; }
   BOT;
   line++;
-  while( isalnum(*line) || (*line == '-') ) {
+  while( isalnum((int)*line) || (*line == '-') ) {
     line++;
   }
   EOT;
