@@ -74,15 +74,15 @@ char *skip_single_char(char *line, char what) {
 
 static __inline__
 char *skip_twodigit(char *line) {
-  if( !line || !isdigit(line[0]) || !isdigit(line[1]) ) { return NULL; }
+  if( !line || !isdigit((int)line[0]) || !isdigit((int)line[1]) ) { return NULL; }
   line += 2;
   return line;
 }
 
 static __inline__
 char *skip_fourdigit(char *line) {
-  if( !line || !isdigit(line[0]) || !isdigit(line[1]) ||
-      !isdigit(line[2]) || !isdigit(line[3]) ) { return NULL; }
+  if( !line || !isdigit((int)line[0]) || !isdigit((int)line[1]) ||
+      !isdigit((int)line[2]) || !isdigit((int)line[3]) ) { return NULL; }
   line += 4;
   return line;
 }
@@ -132,10 +132,10 @@ static __inline__
 char *skip_822_lwsp(char *line, int min) {
   if( !line ) { return NULL; }
   for(; min-- > 0; line++) {
-    if( !isspace(*line) ) { return NULL; }
+    if( !isspace((int)*line) ) { return NULL; }
   }
   while(line) {
-    if( isspace(*line) ) {
+    if( isspace((int)*line) ) {
       line++;
     } else if( *line == '(' ) {
       line = skip_822_comment(line, NULL);
@@ -208,7 +208,7 @@ char *skip_822_seq(char *line, token_delim_t *tok,
 }
 
 static char rfc822_atom[256] = {
-    0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
+   -1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
     0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
     0, '!',   0, '#', '$', '%', '&','\'',   0,   0, '*', '+',   0, '-',   0, '/',
   '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',   0,   0,   0, '=',   0, '?',
@@ -514,8 +514,8 @@ static __inline__
 char *skip_822_zone(char *line, token_delim_t *tok) {
   if( !line ) { return NULL; }
   BOT;
-  if( isalpha(line[0]) ) {
-    if( !isalpha(line[1]) ) {
+  if( isalpha((int)line[0]) ) {
+    if( !isalpha((int)line[1]) ) {
       line++;
     } else {
       line = skip_822_zone_name(line);
@@ -842,9 +842,9 @@ char *parse_822_comments(char *line, parse_822_txt_t *p) {
 
 static __inline__
 char *skip_821_name(char *line, token_delim_t *tok) {
-  if( !line || !isalpha(*line) ) { return NULL; }
+  if( !line || !isalpha((int)*line) ) { return NULL; }
   BOT;
-  while( isalnum(*line) || (*line == '-') ) {
+  while( isalnum((int)*line) || (*line == '-') ) {
     line++;
   }
   EOT;
@@ -854,9 +854,9 @@ char *skip_821_name(char *line, token_delim_t *tok) {
 
 static __inline__
 char *skip_821_number(char *line, token_delim_t *tok) {
-  if( !line || !isdigit(*line) ) { return NULL; }
+  if( !line || !isdigit((int)*line) ) { return NULL; }
   BOT;
-  while( isdigit(*line) ) {
+  while( isdigit((int)*line) ) {
     line++;
   }
   EOT;
@@ -868,21 +868,21 @@ static __inline__
 char *skip_821_dotnum(char *line, token_delim_t *tok) {
   if( !line ) { return NULL; }
   BOT;
-  if( isdigit(*line) ) { line++; } else { return NULL; }
-  if( isdigit(*line) ) { line++; }
-  if( isdigit(*line) ) { line++; }
+  if( isdigit((int)*line) ) { line++; } else { return NULL; }
+  if( isdigit((int)*line) ) { line++; }
+  if( isdigit((int)*line) ) { line++; }
   if( *line != '.' ) { return NULL; }
-  if( isdigit(*line) ) { line++; } else { return NULL; }
-  if( isdigit(*line) ) { line++; }
-  if( isdigit(*line) ) { line++; }
+  if( isdigit((int)*line) ) { line++; } else { return NULL; }
+  if( isdigit((int)*line) ) { line++; }
+  if( isdigit((int)*line) ) { line++; }
   if( *line != '.' ) { return NULL; }
-  if( isdigit(*line) ) { line++; } else { return NULL; }
-  if( isdigit(*line) ) { line++; }
-  if( isdigit(*line) ) { line++; }
+  if( isdigit((int)*line) ) { line++; } else { return NULL; }
+  if( isdigit((int)*line) ) { line++; }
+  if( isdigit((int)*line) ) { line++; }
   if( *line != '.' ) { return NULL; }
-  if( isdigit(*line) ) { line++; } else { return NULL; }
-  if( isdigit(*line) ) { line++; }
-  if( isdigit(*line) ) { line++; }
+  if( isdigit((int)*line) ) { line++; } else { return NULL; }
+  if( isdigit((int)*line) ) { line++; }
+  if( isdigit((int)*line) ) { line++; }
   EOT;
   DT("821_dotnum");
   return line;
